@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2010, 2013, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2010, 2014, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -29,7 +29,7 @@ from datetime import datetime, timedelta
 
 from mysql.utilities.exception import UtilRplError
 from mysql.utilities.common.ip_parser import hostname_is_ip
-from mysql.utilities.common.messages import ERROR_SAME_MASTER
+from mysql.utilities.common.messages import ERROR_SAME_MASTER, HOST_IP_WARNING
 from mysql.utilities.common.tools import ping_host, execute_script
 from mysql.utilities.common.format import print_list
 from mysql.utilities.common.topology import Topology
@@ -76,15 +76,12 @@ _FAILOVER_ERRNO = 911
 _DATE_FORMAT = '%Y-%m-%d %H:%M:%S %p'
 _DATE_LEN = 22
 
-_HOST_IP_WARNING = "You may be mixing host names and IP " + \
-                   "addresses. This may result in negative status " + \
-                   "reporting if your DNS services do not support " + \
-                   "reverse name lookup."
-
 _ERRANT_TNX_ERROR = "Errant transaction(s) found on slave(s)."
 
 _GTID_ON_REQ = "Slave election requires GTID_MODE=ON for all servers."
 
+WARNING_SLEEP_TIME = 10
+
 
 def get_valid_rpl_command_text():
     """Provide list of valid command descriptions to caller.
@@ -119,7 +116,7 @@ def purge_log(filename, age):
         log = open(filename, "r")
         log_entries = log.readlines()
         log.close()
-        threshold = datetime.now() -timedelta(days=age)
+        threshold = datetime.now() - timedelta(days=age)
         start = 0
         for row in log_entries:
             # Check age here
@@ -132,12 +129,12 @@ def purge_log(filename, age):
                     return True
                 else:
                     break
-            except Exception, e:
+            except:
                 start += 1    # Remove invalid formatted lines
         log = open(filename, "w")
         log.writelines(log_entries[start:])
         log.close()
-    except Exception, e:
+    except:
         return False
     return True
 
@@ -187,13 +184,15 @@ class RplCommands(object):
             if daemon != "nodetach":
                 sys.stdout = self.stdout_devnull
 
+        self.master = None
         self.master_vals = master_vals
         self.options = options
         self.quiet = self.options.get("quiet", False)
         self.logging = self.options.get("logging", False)
         self.candidates = self.options.get("candidates", None)
-
+        self.verbose = self.options.get("verbose", None)
         self.rpl_user = self.options.get("rpl_user", None)
+
         try:
             self.topology = Topology(master_vals, slave_vals, self.options,
                                      skip_conn_err)
@@ -252,7 +251,7 @@ class RplCommands(object):
               IO_Thread, SQL_Thread, Secs_Behind, Remaining_Delay,
               IO_Error_Num, IO_Error
         """
-        format = self.options.get("format", "grid")
+        fmt = self.options.get("format", "grid")
         quiet = self.options.get("quiet", False)
 
         cols, rows = self.topology.get_health()
@@ -262,7 +261,7 @@ class RplCommands(object):
             print "# Replication Topology Health:"
 
         # Print health report
-        print_list(sys.stdout, format, cols, rows)
+        print_list(sys.stdout, fmt, cols, rows)
 
         return
 
@@ -274,33 +273,34 @@ class RplCommands(object):
         entries, that list is not printed.
         """
         if not self.topology.gtid_enabled():
-            self._report("# WARNING: GTIDs are not supported on this topology.",
-                         logging.WARN)
+            self._report("# WARNING: GTIDs are not supported on this "
+                         "topology.", logging.WARN)
             return
 
-        format = self.options.get("format", "grid")
+        fmt = self.options.get("format", "grid")
 
         # Get UUIDs
         uuids = self.topology.get_server_uuids()
         if len(uuids):
             print "#"
             print "# UUIDS for all servers:"
-            print_list(sys.stdout, format, ['host','port','role','uuid'], uuids)
+            print_list(sys.stdout, fmt, ['host', 'port', 'role', 'uuid'],
+                       uuids)
 
         # Get GTID lists
         executed, purged, owned = self.topology.get_gtid_data()
         if len(executed):
             print "#"
             print "# Transactions executed on the server:"
-            print_list(sys.stdout, format, _GTID_COLS, executed)
+            print_list(sys.stdout, fmt, _GTID_COLS, executed)
         if len(purged):
             print "#"
             print "# Transactions purged from the server:"
-            print_list(sys.stdout, format, _GTID_COLS, purged)
+            print_list(sys.stdout, fmt, _GTID_COLS, purged)
         if len(owned):
             print "#"
             print "# Transactions owned by another server:"
-            print_list(sys.stdout, format, _GTID_COLS, owned)
+            print_list(sys.stdout, fmt, _GTID_COLS, owned)
 
     def _check_host_references(self):
         """Check to see if using all host or all IP addresses
@@ -317,7 +317,7 @@ class RplCommands(object):
                 if host_port:
                     host = host_port[0]
                 if (not host or uses_ip != hostname_is_ip(slave.host) or
-                    uses_ip != hostname_is_ip(host)):
+                   uses_ip != hostname_is_ip(host)):
                     return False
         return True
 
@@ -332,7 +332,7 @@ class RplCommands(object):
         # Check new master is not actual master - need valid candidate
         candidate = self.options.get("new_master", None)
         if (self.topology.master.is_alias(candidate['host']) and
-            self.master_vals['port'] == candidate['port']):
+           self.master_vals['port'] == candidate['port']):
             err_msg = ERROR_SAME_MASTER.format(candidate['host'],
                                                candidate['port'],
                                                self.master_vals['host'],
@@ -347,8 +347,8 @@ class RplCommands(object):
 
         # Check for mixing IP and hostnames
         if not self._check_host_references():
-            print "# WARNING: %s" % _HOST_IP_WARNING
-            self._report(_HOST_IP_WARNING, logging.WARN, False)
+            print("# WARNING: {0}".format(HOST_IP_WARNING))
+            self._report(HOST_IP_WARNING, logging.WARN, False)
 
         # Check prerequisites
         if candidate is None:
@@ -356,9 +356,35 @@ class RplCommands(object):
             self._report(msg, logging.CRITICAL)
             raise UtilRplError(msg)
 
+        # Can only check errant transactions if GTIDs are enabled.
+        if self.topology.gtid_enabled():
+            # Check existence of errant transactions on slaves
+            errant_tnx = self.topology.find_errant_transactions()
+            if errant_tnx:
+                force = self.options.get('force')
+                print("# ERROR: {0}".format(_ERRANT_TNX_ERROR))
+                self._report(_ERRANT_TNX_ERROR, logging.ERROR, False)
+                for host, port, tnx_set in errant_tnx:
+                    errant_msg = (" - For slave '{0}@{1}': "
+                                  "{2}".format(host, port, ", ".join(tnx_set)))
+                    print("# {0}".format(errant_msg))
+                    self._report(errant_msg, logging.ERROR, False)
+                # Raise an exception (to stop) if tolerant mode is OFF
+                if not force:
+                    raise UtilRplError("{0} Note: If you want to ignore this "
+                                       "issue, although not advised, please "
+                                       "use the utility with the --force "
+                                       "option.".format(_ERRANT_TNX_ERROR))
+        else:
+            warn_msg = ("Errant transactions check skipped (GTID not enabled "
+                        "for the whole topology).")
+            print("# WARNING: {0}".format(warn_msg))
+            self._report(warn_msg, logging.WARN, False)
+
         self._report(" ".join(["# Performing switchover from master at",
                      "%s:%s" % (self.master_vals['host'],
-                     self.master_vals['port']), "to slave at %s:%s." %
+                                self.master_vals['port']),
+                               "to slave at %s:%s." %
                      (candidate['host'], candidate['port'])]))
         if not self.topology.switchover(candidate):
             self._report("# Errors found. Switchover aborted.", logging.ERROR)
@@ -380,8 +406,8 @@ class RplCommands(object):
 
         # Check for mixing IP and hostnames
         if not self._check_host_references():
-            print "# WARNING: %s" % _HOST_IP_WARNING
-            self._report(_HOST_IP_WARNING, logging.WARN, False)
+            print("# WARNING: {0}".format(HOST_IP_WARNING))
+            self._report(HOST_IP_WARNING, logging.WARN, False)
 
         candidates = self.options.get("candidates", None)
         if candidates is None or len(candidates) == 0:
@@ -398,7 +424,7 @@ class RplCommands(object):
         self._report("# Best slave found is located on %s:%s." %
                      (best_slave['host'], best_slave['port']))
 
-    def _failover(self, strict=False, options={}):
+    def _failover(self, strict=False, options=None):
         """Perform failover
 
         This method executes GTID-enabled failover. If called for a non-GTID
@@ -411,6 +437,8 @@ class RplCommands(object):
 
         Returns bool - True = failover succeeded, False = errors found
         """
+        if options is None:
+            options = {}
         srv_list = self.topology.get_servers_with_gtid_not_on()
         if srv_list:
             print("# ERROR: {0}".format(_GTID_ON_REQ))
@@ -440,10 +468,10 @@ class RplCommands(object):
                 self._report(errant_msg, logging.ERROR, False)
             # Raise an exception (to stop) if tolerant mode is OFF
             if not force:
-                raise UtilRplError("%s Note: If you want to ignore this issue,"
-                                   " although not advised, please use the "
-                                   "utility with the --force option."
-                                   % _ERRANT_TNX_ERROR)
+                raise UtilRplError("{0} Note: If you want to ignore this "
+                                   "issue, although not advised, please use "
+                                   "the utility with the --force option."
+                                   "".format(_ERRANT_TNX_ERROR))
 
         self._report("# Performing failover.")
         if not self.topology.failover(self.candidates, strict,
@@ -472,7 +500,12 @@ class RplCommands(object):
                 return False
         return True
 
-    def execute_command(self, command, options={}):
+    def check_host_references(self):
+        """Public method to access self.check_host_references()
+        """
+        return self._check_host_references()
+
+    def execute_command(self, command, options=None):
         """Execute a replication admin command
 
         This method executes one of the valid replication administration
@@ -483,6 +516,8 @@ class RplCommands(object):
 
         Returns bool - True = success, raise error on failure
         """
+        if options is None:
+            options = {}
         # Raise error if command is not valid
         if not command in _VALID_COMMANDS:
             msg = "'%s' is not a valid command." % command
@@ -498,7 +533,7 @@ class RplCommands(object):
                   "execute the %s command."
             for error in errors:
                 self._report(msg % (error[0], error[1], command),
-                                    logging.CRITICAL)
+                             logging.CRITICAL)
             raise UtilRplError("Not enough privileges to execute command.")
 
         self._report("Executing %s command..." % command, logging.INFO, False)
@@ -554,6 +589,19 @@ class RplCommands(object):
                                   self.topology.get_server_uuids,
                                   self.options)
 
+        # Check privileges
+        self._report("# Checking privileges.")
+        errors = self.topology.check_privileges(failover_mode != 'fail')
+        if len(errors):
+            for error in errors:
+                msg = ("User {0} on {1}@{2} does not have sufficient "
+                       "privileges to execute the {3} command "
+                       "(required: {4}).").format(error[0], error[1], error[2],
+                                                  'failover', error[3])
+                print("# ERROR: {0}".format(msg))
+                self._report(msg, logging.CRITICAL, False)
+            raise UtilRplError("Not enough privileges to execute command.")
+
         # Unregister existing instances from slaves
         self._report("Unregistering existing instances from slaves.",
                      logging.INFO, False)
@@ -572,15 +620,17 @@ class RplCommands(object):
             print "Failover mode changed to 'FAIL' for this instance. "
             print "Console will start in 10 seconds.",
             sys.stdout.flush()
-            for i in range(0, 9):
+            i = 0
+            while i < 9:
                 time.sleep(1)
                 sys.stdout.write('.')
                 sys.stdout.flush()
+                i += 1
             print "starting Console."
             time.sleep(1)
 
         try:
-            res = self.run_auto_failover(console, interval, failover_mode)
+            res = self.run_auto_failover(console, failover_mode)
         except:
             raise
         finally:
@@ -632,7 +682,7 @@ class RplCommands(object):
 
         return res
 
-    def run_auto_failover(self, console, interval, failover_mode="auto"):
+    def run_auto_failover(self, console, failover_mode="auto"):
         """Run automatic failover
 
         This method implements the automatic failover facility. It uses the
@@ -646,13 +696,11 @@ class RplCommands(object):
         2) failover to list of candidates only
         3) fail
 
-        console[in]    instance of the failover console class
-        interval[in]   time in seconds to wait to check status of servers
+        console[in]    instance of the failover console class.
 
         Returns bool - True = success, raises exception on error
         """
         pingtime = self.options.get("pingtime", 3)
-        timeout = int(self.options.get("timeout", 300))
         exec_fail = self.options.get("exec_fail", None)
         post_fail = self.options.get("post_fail", None)
         pedantic = self.options.get('pedantic', False)
@@ -664,19 +712,6 @@ class RplCommands(object):
             self._report(msg, logging.CRITICAL)
             raise UtilRplError(msg)
 
-        # Check privileges
-        self._report("# Checking privileges.")
-        errors = self.topology.check_privileges(failover_mode != 'fail')
-        if len(errors):
-            for error in errors:
-                msg = ("User {0} on {1}@{2} does not have sufficient "
-                       "privileges to execute the {3} command "
-                       "(required: {4}).").format(error[0], error[1], error[2],
-                                                  'failover', error[3])
-                print("# ERROR: {0}".format(msg))
-                self._report(msg, logging.CRITICAL, False)
-            raise UtilRplError("Not enough privileges to execute command.")
-
         # Require --master-info-repository=TABLE for all slaves
         if not self.topology.check_master_info_type("TABLE"):
             msg = "Failover requires --master-info-repository=TABLE for " + \
@@ -686,10 +721,11 @@ class RplCommands(object):
 
         # Check for mixing IP and hostnames
         if not self._check_host_references():
-            print "# WARNING: %s" % _HOST_IP_WARNING
-            self._report(_HOST_IP_WARNING, logging.WARN, False)
-            print "#\n# Failover console will start in 10 seconds."
-            time.sleep(10)
+            print("# WARNING: {0}".format(HOST_IP_WARNING))
+            self._report(HOST_IP_WARNING, logging.WARN, False)
+            print("#\n# Failover console will start in {0} seconds.".format(
+                WARNING_SLEEP_TIME))
+            time.sleep(WARNING_SLEEP_TIME)
 
         # Test failover script. If it doesn't exist, fail.
         no_exec_fail_msg = "Failover check script cannot be found. Please " + \
@@ -716,7 +752,8 @@ class RplCommands(object):
                                    "option.".format(_ERRANT_TNX_ERROR))
 
         self._report("Failover console started.", logging.INFO, False)
-        self._report("Failover mode = %s." % failover_mode, logging.INFO, False)
+        self._report("Failover mode = %s." % failover_mode, logging.INFO,
+                     False)
 
         # Main loop - loop and fire on interval.
         done = False
@@ -728,7 +765,6 @@ class RplCommands(object):
                 old_host = self.master.host
                 old_port = self.master.port
             except:
-                pass
                 old_host = "UNKNOWN"
                 old_port = "UNKNOWN"
 
@@ -763,16 +799,30 @@ class RplCommands(object):
                     try:
                         self.topology.master.connect()
                     except:
-                        self._report("Cannot reconnect to master.",
-                                     logging.INFO, False)
+                        pass
 
                 # Check the master again. If no connection or lost connection,
-                # try ping and if still not alive, failover. This performs the
-                # timeout threshold for detecting a down master.
+                # try ping. This performs the timeout threshold for detecting
+                # a down master. If still not alive, try to reconnect and if
+                # connection fails after 3 attempts, failover.
                 if self.topology.master is None or \
                    not ping_host(self.topology.master.host, pingtime) or \
                    not self.topology.master.is_alive():
                     failover = True
+                    i = 0
+                    while i < 3:
+                        try:
+                            self.topology.master.connect()
+                            failover = False  # Master is now connected again
+                            break
+                        except:
+                            pass
+                        time.sleep(pingtime)
+                        i += 1
+
+                    if failover:
+                        self._report("Failed to reconnect to the master after "
+                                     "3 attemps.", logging.INFO)
 
             if failover:
                 self._report("Master is confirmed to be down or unreachable.",
@@ -789,16 +839,16 @@ class RplCommands(object):
                     self._report("Failover starting in 'elect' mode...")
                     res = self.topology.failover(self.candidates, True)
                 else:
-                    msg = _FAILOVER_ERROR % \
-                          "Master has failed and automatic failover is not enabled. "
+                    msg = _FAILOVER_ERROR % ("Master has failed and automatic "
+                                             "failover is not enabled. ")
                     self._report(msg, logging.CRITICAL, False)
                     # Execute post failover script
                     self.topology.run_script(post_fail, False,
                                              [old_host, old_port])
                     raise UtilRplError(msg, _FAILOVER_ERRNO)
                 if not res:
-                    msg = _FAILOVER_ERROR % "An error was encountered " + \
-                          "during failover. "
+                    msg = _FAILOVER_ERROR % ("An error was encountered "
+                                             "during failover. ")
                     self._report(msg, logging.CRITICAL, False)
                     # Execute post failover script
                     self.topology.run_script(post_fail, False,
@@ -829,8 +879,8 @@ class RplCommands(object):
                 failover_mode = console.register_instance()
 
             # discover slaves if option was specified at startup
-            elif self.options.get("discover", None) is not None and \
-                (not first_pass or self.options.get("rediscover", False)):
+            elif (self.options.get("discover", None) is not None
+                  and not first_pass):
                 # Force refresh of health list if new slaves found
                 if self.topology.discover_slaves():
                     console.list_data = None
