@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2010, 2013, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2010, 2014, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -411,6 +411,52 @@ class Database(object):
                 )
         return create_str
 
+    def _get_views_sorted_by_dependencies(self, views, columns,
+                                          need_backtick=True):
+        """Get a list of views sorted by their dependencies.
+
+        views[in]          List of views objects
+        columns[in]        Column mode - names (default), brief, or full
+        need_backtick[in]  True if view need backticks in the name
+
+        Returns the list of view sorted by their dependencies
+        """
+        if columns == "names":
+            name_idx = 0
+        elif columns == "full":
+            name_idx = 2
+        else:
+            name_idx = 1
+
+        def views_cmp(v1, v2):
+            """Custom comparison function of two views
+
+            This method tries to find v1 name in v2 definition, if found means
+            that there is dependency (v2 depends of v1).
+
+            v1[in]  Tuple containing the columns of the first view
+            v2[in]  Tuple containing the columns of the second view
+
+            Returns -1 if v1 name is in v2 definition.
+            """
+            # Get the create statement of the view
+            stmt = self.get_create_statement(self.db_name, v2[name_idx], _VIEW)
+            # Name of the view with backticks to be found
+            search = quote_with_backticks(v1[name_idx]) \
+                if need_backtick else v1[name_idx]
+            index = stmt.find(search)
+            if index >= 0:
+                return -1  # Found
+            else:
+                return 0  # Not found
+
+        # Returns without columns names
+        if isinstance(views[0], tuple):
+            return sorted(views, cmp=views_cmp)
+
+        # Returns the tuple reconstructed with views sorted
+        return (views[0], sorted(views[1], cmp=views_cmp),)
+
     def __add_db_objects(self, obj_type):
         """Get a list of objects from a database based on type.
 
@@ -716,7 +762,7 @@ class Database(object):
         name by setting new_db = old_db or as a new database.
 
         new_db[in]         Name of the new database
-        options[in]        Options for copy e.g. force, etc.
+        options[in]        Options for copy e.g. do_drop, etc.
         new_server[in]     Connection to another server for copying the db
                            Default is None (copy to same server - clone)
         connections[in]    Number of threads(connections) to use for insert
@@ -761,11 +807,11 @@ class Database(object):
                 exists = self.exists(self.destination, new_db)
                 drop_server = self.destination
             if exists:
-                if options.get("force", False):
+                if options.get("do_drop", False):
                     self.drop(drop_server, True, new_db)
                 elif not self.skip_create:
                     raise UtilDBError("destination database exists. Use "
-                                      "--force to overwrite existing "
+                                      "--drop-first to overwrite existing "
                                       "database.", -1, new_db)
 
         db_name = self.db_name
@@ -782,9 +828,9 @@ class Database(object):
 
         # Create the objects in the new database
         for obj in self.objects:
-            # Drop object if --force specified and database not dropped
+            # Drop object if --drop-first specified and database not dropped
             # Grants do not need to be dropped for overwriting
-            if options.get("force", False) and obj[0] != _GRANT:
+            if options.get("do_drop", False) and obj[0] != _GRANT:
                 obj_name = quote_with_backticks(obj[1][0])
                 self.__drop_object(obj[0], obj_name)
 
@@ -809,7 +855,7 @@ class Database(object):
         if the option was selected by the caller.
 
         new_db[in]          Name of the new database
-        options[in]         Options for copy e.g. force, etc.
+        options[in]         Options for copy e.g. do_drop, etc.
         new_server[in]      Connection to another server for copying the db
                             Default is None (copy to same server - clone)
         connections[in]     Number of threads(connections) to use for insert
@@ -1500,6 +1546,10 @@ class Database(object):
                 # set new result with with required data quoted with backticks
                 res = (res[0], new_rows)
 
+            if res and obj_type == _VIEW:
+                res = self._get_views_sorted_by_dependencies(res, columns,
+                                                             not need_backtick)
+
             return res
 
     def _check_user_permissions(self, uname, host, access):
@@ -1555,6 +1605,10 @@ class Database(object):
         if not options.get('skip_events', False):
             priv_tuple = (self.db_name, "EVENT")
             source_privs.append(priv_tuple)
+        # if triggers, we need trigger
+        if not options.get('skip_triggers', False):
+            priv_tuple = (self.db_name, "TRIGGER")
+            source_privs.append(priv_tuple)
 
         # Check permissions on source
         for priv in source_privs:
@@ -1567,7 +1621,8 @@ class Database(object):
 
         return True
 
-    def check_write_access(self, user, host, options):
+    def check_write_access(self, user, host, options, source_objects=None,
+                           do_drop=False):
         """Check access levels for creating and writing database objects
 
         This method will check the user's permission levels for copying a
@@ -1584,18 +1639,128 @@ class Database(object):
             skip_func      True = no functions processed
             skip_grants    True = no grants processed
             skip_events    True = no events processed
-
+        source_objects[in] Dictionary containing the list of objects from
+                           source database
+        do_drop[in]        True if the user is using --drop-first option
 
         Returns True if user has permissions and raises a UtilDBError if the
                      user does not have permission with a message that includes
                      the server context.
         """
+        if source_objects is None:
+            source_objects = {}
 
         dest_privs = [(self.db_name, "CREATE"),
-                      (self.db_name, "SUPER"),
-                      ("*", "SUPER")]
+                      (self.db_name, "ALTER"),
+                      (self.db_name, "SELECT"),
+                      (self.db_name, "INSERT"),
+                      (self.db_name, "UPDATE"),
+                      (self.db_name, "LOCK TABLES")]
+
+        # Check for the --drop-first
+        if do_drop:
+            dest_privs.append((self.db_name, "DROP"))
+
+        extra_privs = []
+        super_needed = False
+
+        try:
+            res = self.source.exec_query("SELECT CURRENT_USER()")
+            dest_user = res[0][0]
+        except UtilError as err:
+            raise UtilError("Unable to execute SELECT current_user(). Error: "
+                            "{0}".format(err.errmsg))
+
+        # CREATE VIEW is needed for views
+        if not options.get("skip_views", False):
+            views = source_objects.get("views", None)
+            if views:
+                extra_privs.append("CREATE VIEW")
+                for item in views:
+                    # Test if DEFINER is equal to the current user
+                    if item[6] != dest_user:
+                        super_needed = True
+                        break
+
+        # CREATE ROUTINE and EXECUTE are needed for procedures
+        if not options.get("skip_procs", False):
+            procs = source_objects.get("procs", None)
+            if procs:
+                extra_privs.append("CREATE ROUTINE")
+                extra_privs.append("EXECUTE")
+                if not super_needed:
+                    for item in procs:
+                        # Test if DEFINER is equal to the current user
+                        if item[11] != dest_user:
+                            super_needed = True
+                            break
+
+        # CREATE ROUTINE and EXECUTE are needed for functions
+        if not options.get("skip_funcs", False):
+            funcs = source_objects.get("funcs", None)
+            if funcs:
+                if "CREATE ROUTINE" not in extra_privs:
+                    extra_privs.append("CREATE ROUTINE")
+                if "EXECUTE" not in extra_privs:
+                    extra_privs.append("EXECUTE")
+                if not super_needed:
+                    trust_function_creators = False
+                    try:
+                        res = self.source.show_server_variable(
+                            "log_bin_trust_function_creators"
+                        )
+                        if res and isinstance(res, list) and \
+                                res[0][1] in ("ON", "1"):
+                            trust_function_creators = True
+                        # If binary log is enabled and
+                        # log_bin_trust_function_creators is 0, we need
+                        # SUPER privilege
+                        super_needed = self.source.binlog_enabled() and \
+                            not trust_function_creators
+                    except UtilError as err:
+                        raise UtilDBError("ERROR: {0}".format(err.errmsg))
+
+                    if not super_needed:
+                        for item in funcs:
+                            # Test if DEFINER is equal to the current user
+                            if item[11] != dest_user:
+                                super_needed = True
+                                break
+
+        # EVENT is needed for events
+        if not options.get("skip_events", False):
+            events = source_objects.get("events", None)
+            if events:
+                extra_privs.append("EVENT")
+                if not super_needed:
+                    for item in events:
+                        # Test if DEFINER is equal to the current user
+                        if item[3] != dest_user:
+                            super_needed = True
+                            break
+
+        # TRIGGER is needed for events
+        if not options.get("skip_triggers", False):
+            triggers = source_objects.get("triggers", None)
+            if triggers:
+                extra_privs.append("TRIGGER")
+                if not super_needed:
+                    for item in triggers:
+                        # Test if DEFINER is equal to the current user
+                        if item[18] != dest_user:
+                            super_needed = True
+                            break
+
+        # Add SUPER privilege if needed
+        if super_needed:
+            dest_privs.append(("*", "SUPER"))
+
+        # Add extra privileges needed
+        for priv in extra_privs:
+            dest_privs.append((self.db_name, priv))
+
         if not options.get('skip_grants', False):
-            priv_tuple = (self.db_name, "WITH GRANT OPTION")
+            priv_tuple = (self.db_name, "GRANT OPTION")
             dest_privs.append(priv_tuple)
 
         # Check privileges on destination
