@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2013 Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2013, 2014 Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,6 +20,10 @@ This file contains output string messages used by MySQL Utilities.
 """
 
 
+ERROR_USER_WITHOUT_PRIVILEGES = ("User '{user}' on '{host}@{port}' does not "
+                                 "have sufficient privileges to perform "
+                                 "{operation} (required: {req_privileges}).")
+
 PARSE_ERR_DB_PAIR = ("Cannot parse the specified database(s): '{db_pair}'. "
                      "Please verify that the database(s) are specified in "
                      "a valid format (i.e., {db1_label}[:{db2_label}]) and "
@@ -61,27 +65,47 @@ PARSE_ERR_DB_OBJ_MISSING = ("No object has been specified for "
 
 PARSE_ERR_DB_MISSING_CMP = "No databases specified to compare."
 
+PARSE_ERR_OBJ_NAME_FORMAT = ("Cannot parse the specified qualified name "
+                             "'{obj_name}' for {option}. Please verify that a "
+                             "valid format is used (i.e., <db_name>"
+                             "[.<tbl_name>]) and that backtick quotes are "
+                             "properly used if required.")
+
 PARSE_ERR_SPAN_KEY_SIZE_TOO_LOW = (
     "The value {s_value} specified for option --span-key-size is too small "
     "and would cause inaccurate results, please retry with a bigger value "
     "or the default value of {default}.")
 
-PARSE_ERR_OPT_INVALID_CMD = ("Invalid {opt} option for '{cmd}'.")
+PARSE_ERR_OPT_INVALID_CMD = "Invalid {opt} option for '{cmd}'."
 
 PARSE_ERR_OPT_INVALID_CMD_TIP = ("%s Use {opt_tip} instead."
                                  % PARSE_ERR_OPT_INVALID_CMD)
 
+PARSE_ERR_OPT_REQ_NON_NEGATIVE_VALUE = ("Option '{opt}' requires a "
+                                        "non-negative value.")
+
+PARSE_ERR_OPT_REQ_GREATER_VALUE = ("Option '{opt}' requires a value greater "
+                                   "than {val}.")
+
+PARSE_ERR_OPT_REQ_VALUE = "Option '{opt}' requires a non-empty value."
+
 PARSE_ERR_OPTS_REQ = "Option '{opt}' is required."
 
 PARSE_ERR_OPTS_REQ_BY_CMD = ("'{cmd}' requires the following option(s): "
                              "{opts}.")
 
+PARSE_ERR_DISCO_REQ_MASTER = ("Option --discover-slaves-login requires use of "
+                              "--master.")
+
 PARSE_ERR_SLAVE_DISCO_REQ = ("Option --discover-slaves-login or --slaves is "
                              "required.")
 
 PARSE_ERR_SLAVE_DISCO_EXC = ("Options --discover-slaves-login and --slaves "
                              "cannot be used simultaneously.")
 
+PARSE_ERR_OPTS_REQ_GREATER_OR_EQUAL = ("The {opt} option requires a value "
+                                       "greater than or equal to {value}.")
+
 WARN_OPT_NOT_REQUIRED = ("WARNING: The {opt} option is not required for "
                          "'{cmd}' (option ignored).")
 
@@ -111,3 +135,11 @@ SCRIPT_THRESHOLD_WARNING = ("WARNING: You have chosen to use external script "
                             "fails, this can leave the operation in an "
                             "undefined state. Please check your results "
                             "carefully if the operation aborts.")
+
+HOST_IP_WARNING = ("You may be mixing host names and IP addresses. This may "
+                   "result in negative status reporting if your DNS services "
+                   "do not support reverse name lookup.")
+
+ERROR_MIN_SERVER_VERSIONS = ("The {utility} requires server versions greater "
+                             "or equal than {min_version}. Server version for "
+                             "'{host}:{port}' is not supported.")
