@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 #
-# Copyright (c) 2010, 2013, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2010, 2014, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -44,7 +44,8 @@ from mysql.utilities.common.options import (add_skip_options, add_verbosity,
                                             add_all, check_all, add_locking,
                                             add_regexp, add_rpl_mode,
                                             add_rpl_user, setup_common_options,
-                                            add_character_set_option)
+                                            add_character_set_option,
+                                            check_password_security)
 from mysql.utilities.common.sql_transform import (is_quoted_with_backticks,
                                                   remove_backtick_quoting)
 from mysql.utilities.common.tools import (check_connector_python,
@@ -90,9 +91,9 @@ if __name__ == '__main__':
     add_character_set_option(parser)
 
     # Overwrite mode
-    parser.add_option("-f", "--force", action="store_true", dest="force",
+    parser.add_option("-d", "--drop-first", action="store_true", default=False,
                       help="drop the new database or object if it exists",
-                      default=False)
+                      dest="do_drop")
 
     # Add the exclude database option
     parser.add_option("-x", "--exclude", action="append", dest="exclude",
@@ -123,7 +124,7 @@ if __name__ == '__main__':
     add_regexp(parser)
 
     # Replication user and password
-    add_rpl_user(parser, None)
+    add_rpl_user(parser)
 
     # Add replication options but don't include 'both'
     add_rpl_mode(parser, False, False)
@@ -143,6 +144,9 @@ if __name__ == '__main__':
     # Now we process the rest of the arguments.
     opt, args = parser.parse_args()
 
+    # Check security settings
+    check_password_security(opt, args)
+
     try:
         skips = check_skip_options(opt.skip_objects)
     except UtilError:
@@ -201,7 +205,7 @@ if __name__ == '__main__':
         "skip_grants": "grants" in skips,
         "skip_create": "create_db" in skips,
         "skip_data": "data" in skips,
-        "force": opt.force,
+        "do_drop": opt.do_drop,
         "verbose": opt.verbosity >= 1,
         "quiet": opt.quiet,
         "debug": opt.verbosity == 3,
